from __future__ import unicode_literals
try:
    from unittest import skipUnless
except ImportError:  # Python 2.6
    from django.utils.unittest import skipUnless

from django import VERSION
from django.core.exceptions import ImproperlyConfigured
from django.test import TestCase
from django.test.utils import override_settings
from django.utils.six import text_type
from django.contrib.auth.models import User
from django.core.mail import send_mail
from django.core import mail

from email_log.models import Email
from email_log.conf import Settings

FAILING_BACKEND = 'email_log.tests.backends.FailingEmailBackend'


class EmailModelTests(TestCase):

    def test_unicode(self):
        email = Email.objects.create(
            from_email="from@example.com",
            recipients="to@example.com",
            subject="Subject line",
            body="Message body",
            ok=True,
        )
        self.assertEqual(text_type(email), "to@example.com: Subject line")


class SettingsTests(TestCase):

    @override_settings(SPECIFIED_SETTING="specified")
    def test_settings_fall_through(self):
        settings = Settings()
        settings.defaults.UNSPECIFIED_SETTING = "unspecified"
        self.assertEqual(settings.SPECIFIED_SETTING, "specified")

    def test_settings_defaults(self):
        settings = Settings()
        settings.defaults.UNSPECIFIED_SETTING = "unspecified"
        self.assertEqual(settings.UNSPECIFIED_SETTING, "unspecified")


@override_settings(EMAIL_BACKEND='email_log.backends.EmailBackend')
class EmailBackendTests(TestCase):

    plain_args = {
        'from_email': "from@example.com",
        'recipients': ["to@example.com"],
        'subject': "Subject line",
        'body': "Message body",
    }

    def assertEmailCorrect(self, email, model=True, **kwargs):
        for key, value in kwargs.items():
            if key == 'recipients' and model:
                value = '; '.join(value)
            self.assertEqual(getattr(email, key), value)

    def send_mail(self, **kwargs):
        kwgs = dict(**kwargs)
        args = [kwgs.pop(k) for k in
                ('subject', 'body', 'from_email', 'recipients')]
        return send_mail(*args, **kwgs)

    def test_send_messages(self):
        sent = self.send_mail(fail_silently=False, **self.plain_args)
        self.assertEqual(sent, 1)
        self.assertEqual(len(mail.outbox), 1)
        email = Email.objects.get()
        self.assertTrue(email.ok)
        self.assertEqual(email.recipients, "to@example.com")
        self.assertSequenceEqual(mail.outbox[0].to, ["to@example.com"])
        for message in (email, mail.outbox[0]):
            self.assertEqual(message.from_email, "from@example.com")
            self.assertEqual(message.subject, "Subject line")
            self.assertEqual(message.body, "Message body")

    @override_settings(EMAIL_LOG_BACKEND=FAILING_BACKEND)
    def test_send_failure(self):
        self.assertRaises(NotImplementedError, self.send_mail,
                          fail_silently=False, **self.plain_args)
        email = Email.objects.get()
        self.assertFalse(email.ok)
        self.assertEmailCorrect(email, **self.plain_args)

    @override_settings(EMAIL_LOG_BACKEND=FAILING_BACKEND)
    def test_send_failure_silent(self):
        sent = self.send_mail(fail_silently=True, **self.plain_args)
        self.assertEqual(sent, 0)
        email = Email.objects.get()
        self.assertFalse(email.ok)
        self.assertEmailCorrect(email, **self.plain_args)


class AdminTests(TestCase):

    def setUp(self):
        user = User(username="user", is_superuser=True, is_staff=True)
        user.set_password("pass")
        user.save()
        self.client.login(username="user", password="pass")

    def test_add_page(self):
        page = self.client.get('/admin/email_log/email/add/')
        self.assertEqual(page.status_code, 403)

    def test_body_is_formatted(self):
        initial = b"This\nis\na\ntest"
        email = Email.objects.create(body=initial)
        page = self.client.get('/admin/email_log/email/%s/' % email.pk)
        self.assertNotIn(b'<div class="form-row field-body">', page.content)
        self.assertNotIn(initial, page.content)
        self.assertIn(b'<div class="form-row field-body_formatted">',
                      page.content)
        self.assertIn(b'<p>This<br />is<br />a<br />test</p>', page.content)
        self.assertEqual(page.status_code, 200)

    def test_delete_page(self):
        email = Email.objects.create()
        page = self.client.get('/admin/email_log/email/{0}/delete/'
                               .format(email.pk))
        self.assertEqual(page.status_code, 403)

    def test_app_name(self):
        page = self.client.get('/admin/')
        if VERSION < (1, 7, 0):
            self.assertContains(page, "Email_Log")
        else:
            self.assertContains(page, "Email log")


class SouthSupportTests(TestCase):

    @skipUnless(VERSION < (1, 7, 0), "test only applies to 1.6 and below")
    def test_import_migrations_module(self):
        try:
            from email_log.migrations import __doc__  # noqa
        except ImproperlyConfigured as e:
            exception = e
        self.assertIn("SOUTH_MIGRATION_MODULES", exception.args[0])



from __future__ import unicode_literals

from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _


@python_2_unicode_compatible
class Email(models.Model):

    """Model to store outgoing email information"""

    from_email = models.TextField(_("from e-mail"))
    recipients = models.TextField(_("recipients"))
    subject = models.TextField(_("subject"))
    body = models.TextField(_("body"))
    ok = models.BooleanField(_("ok"), default=False, db_index=True)
    date_sent = models.DateTimeField(_("date sent"), auto_now_add=True,
                                     db_index=True)

    def __str__(self):
        return "{s.recipients}: {s.subject}".format(s=self)

    class Meta:
        verbose_name = _("e-mail")
        verbose_name_plural = _("e-mails")
        ordering = ('-date_sent',)



# encoding: utf8
from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Email',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False,
                                        auto_created=True, primary_key=True)),
                ('from_email', models.TextField(verbose_name='from e-mail')),
                ('recipients', models.TextField(verbose_name='recipients')),
                ('subject', models.TextField(verbose_name='subject')),
                ('body', models.TextField(verbose_name='body')),
                ('ok', models.BooleanField(default=False, db_index=True,
                                           verbose_name='ok')),
                ('date_sent', models.DateTimeField(auto_now_add=True,
                                                   verbose_name='date sent',
                                                   db_index=True)),
            ],
            options={
                'ordering': ('-date_sent',),
                'verbose_name': 'e-mail',
                'verbose_name_plural': 'e-mails',
            },
            bases=(models.Model,),
        ),
    ]



from django.conf import settings as defined_settings


class Settings:

    """Class used for allowing default settings for app"""

    class Default:
        EMAIL_LOG_BACKEND = "django.core.mail.backends.smtp.EmailBackend"

    def __init__(self):
        self.defaults = Settings.Default()

    def __getattr__(self, name):
        try:
            return getattr(defined_settings, name)
        except AttributeError:
            return getattr(self.defaults, name)


settings = Settings()



from django.core.mail.backends.base import BaseEmailBackend


class FailingEmailBackend(BaseEmailBackend):

    """Email backend which always fails to send email"""

    def send_messages(self, email_messages):
        try:
            super(FailingEmailBackend, self).send_messages(email_messages)
        except NotImplementedError:
            if not self.fail_silently:
                raise
            return 0



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Email'
        db.create_table(u'email_log_email', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('from_email', self.gf('django.db.models.fields.TextField')()),
            ('recipients', self.gf('django.db.models.fields.TextField')()),
            ('subject', self.gf('django.db.models.fields.TextField')()),
            ('body', self.gf('django.db.models.fields.TextField')()),
            ('ok', self.gf('django.db.models.fields.BooleanField')(default=False, db_index=True)),
            ('date_sent', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, db_index=True, blank=True)),
        ))
        db.send_create_signal(u'email_log', ['Email'])


    def backwards(self, orm):
        # Deleting model 'Email'
        db.delete_table(u'email_log_email')


    models = {
        u'email_log.email': {
            'Meta': {'ordering': "(u'-date_sent',)", 'object_name': 'Email'},
            'body': ('django.db.models.fields.TextField', [], {}),
            'date_sent': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'db_index': 'True', 'blank': 'True'}),
            'from_email': ('django.db.models.fields.TextField', [], {}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'ok': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'db_index': 'True'}),
            'recipients': ('django.db.models.fields.TextField', [], {}),
            'subject': ('django.db.models.fields.TextField', [], {})
        }
    }

    complete_apps = ['email_log']